! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! 
!.IDENTIFICATION: ccd_standard_test.prg    
!.PURPOSE:        Procedure to reduce a standard test
!.AUTHOR:         P. Sinclaire
!.USE:            @@ ccd_standard_test p1 p2 p3 p4 p5 p6
!.                where p1 = number of CCD
!.                      p2 = date of test (format YYMMDD)
!.                      p3 = image filename prefix
!.                      p4 = readout mode
!.                      p5 = file number of last image
!.                      p6 = Number of images incl. the 5 bias
!.Algorithm:      Using a set of 5 bias frames, and n pairs of flat
!                 fields the linearity of a CCD is tested. 
!.VERSION: 
! 980706	creation PSI
! 100321	last modif
!      
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! 
DEFINE/PARAMETER P1 ?            C*2  "Number of CCD:"
DEFINE/PARAMETER P2 "000000"     C*6  "Date of test (YYYYMMDD):"
DEFINE/PARAMETER P3 "test"       C    "Image filename prefix:"
DEFINE/PARAMETER P4 "slow"       C    "CCD readout mode:"
DEFINE/PARAMETER P5 1            N    "File number of last image:"
DEFINE/PARAMETER P6 25           N    "Total number of images:"


write/out " "
write/out " "
write/out "   You have just started the linearity reduction package. The"
write/out "   25 images must be in the same order as taken by the CCD"
write/out "   ccd_standardtest test script."
write/out " "
write/out "   This procedure assumes that you run Midas version 95NOV."
write/out " "
write/out "   During execution of this procedure the Midas display window"
write/out "   should be completely inside the area of your workstation,"
write/out "   and not covered by any other display window. Also, do not"
write/out "   inconify this window, or move to another workspace during"
write/out "   execution."
write/out " "
write/out "   All the files created by this procedure will have the suffix"
write/out "   _{p1}_{p2}."
write/out " "
write/out "   At the end of the test the procedure will automatically print"
write/out "   out the test results and images at the assigned printer. A"
write/out "   copy of this test report should be delivered to the ODT team."
write/out " "
write/out "   In case of problems with the reduction package contact Peter"
write/out "   Sinclaire (psinclai@eso.org), or any member of the ODT team."
write/out " "
write/out "                         Good luck!"
write/out " "
! 
! keywords used for comparisons of actual data with reference data
write/keyw errsum/i/1/1 0
write/keyw icorr/i/1/20 0 all
write/keyw rcorr/r/1/20 0. all
write/keyw dcorr/d/1/20 0. all
!
! Definition of keywords used in this programma
!
define/local dt/d/1/1 0.0
define/local t0/d/1/1 0.0
! 
define/local firstH/i/1/1 0
define/local lastH/i/1/1 0
define/local firstV/i/1/1 0
define/local lastV/i/1/1 0
define/local deltaH/i/1/1 0
define/local deltaV/i/1/1 0
define/local shutLOW/r/1/1 0.0
define/local shutHIGH/r/1/1 0.5
!
! Store frames of the different tests in appropriate image catalogs, define the
! window in which the test are taken and fill the IDENT and O_TIME descriptors
!
! descriptor O_TIME(5): UT time start not filled yet??? OCt96/PS
! in New VLT SW, start time is O_TIME(4)   Aug.99/PS
! 
define/local catal/i/1/1 0
define/local nfrm/i/1/1 0
define/local minmax/r/1/2 0.0 0.0
define/local lastf/i/1/1 0
define/local firsf/i/1/1 0
! 
lastf = {p5}
firsf = {lastf} - {p6} + 1
! 
write/key detec/C/1/20 CCD#{p1}
! 
set/format i4
write/out "    Preparing image catalogs. This may take a while..."
write/out "    processing files: {p3}{firsf}.bdf to {p3}{lastf}.bdf"
! 
reset/disp
! 
! create display first in case not enough color, private Lut is loaded
crea/grap 0 620,480,800,70
crea/disp 0 550,510,50,30
! 
! *** check if the catalogue exists
IF M$EXIST("Std_{p4}_{p1}_{p2}.cat") .eq. 0 THEN 
   create/icat Std_{p4}_{p1}_{p2}.cat NULL
   add/icat Std_{p4}_{p1}_{p2} {p3}*.bdf {p3}{firsf}.bdf,{p3}{lastf}.bdf
   show/icat Std_{p4}_{p1}_{p2}.cat
   if outputi(1) .ne. {p6} .or. outputi(2) .ne. {p6} then
      WRITE/OUT "FATAL error: Number of images does not match."
      Write/out "             check catalog Std_{p4}_{p1}_{p2}.cat"
      write/out ****** procedure ccd_standard_test failed!
      RETURN/EXIT
   endif
ENDIF

nfrm = 0

LOOP1:
   store/frame in_a Std_{p4}_{p1}_{p2}.cat 1 CONT1

   t0 = -1.0
   copy/dk {in_a} O_TIME/d/7/1 dt 
   copy/dk {in_a} TM_START t0        ! TM_START could double or integer
   if t0 .lt. 0.0 then
        write/out "Calculating Start UTC from O_TIME(4)"
        copy/dk {in_a} O_TIME/d/4/1 t0
   endif
   t0 = 24.0d0 * ( t0 - m$nint(t0-0.5) )
   copy/kd t0/d/1/1 {in_a} O_TIME/d/5/1

   set/format ,f6.1

   write/key tc/c/1/5 {dt}

   nfrm = nfrm + 1

   if nfrm .eq. 1 then
       copy/dk {in_a} START/d/1/1 firstH
       copy/dk {in_a} START/d/2/1 firstV
       ! 
       copy/dk {in_a} NPIX/i/1/1 deltaH
       copy/dk {in_a} NPIX/i/2/1 deltaV
       ! 
       lastH = firstH + deltaH - 1
       lastV = firstV + deltaV - 1
       ! 
       write/descr {in_a} ident/c/1/60 {p1}/BIAS:{tc}
       crea/icat Bias_{p4}_{p1}_{p2} {in_a}
    endif
! 
   if nfrm .gt. 1 .and. nfrm .lt. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/BIAS:{tc}
       add/icat Bias_{p4}_{p1}_{p2} {in_a}
    endif
! 
   if nfrm .eq. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/FF:{tc}
       crea/icat FF_{p4}_{p1}_{p2} {in_a}
    endif
   if nfrm .gt. 6 .and. nfrm .lt. 26 then
       write/descr {in_a} ident/c/1/60 {p1}/FF:{tc}
       add/icat FF_{p4}_{p1}_{p2} {in_a}
   endif
   goto LOOP1

CONT1:

set/format

! Initializing CCD context

clear/context -all
set/context ccdtest

! Clear logfile and set to 60 lines per page
log(7) = 60

write/out " "
write/out " "
write/out "     ===================================================="
write/out "                STANDARD TEST REDUCTION PACKAGE"
write/out "     ===================================================="
write/out " "
write/out " "
write/out " "
write/out "                CCD #{P1}  MODE: {p4}"
write/out " "
write/out " "
write/out "                      DATE TEST {P2}"
write/out " "

@@ ccdtestb ? Bias_{p4}_{p1}_{p2}.cat Std_{p4}_{p1}_{p2} median
if errsum .gt. 0 then
   write/out ****** procedure ccd_standard_test failed:
   write/out ****** problems with ccdtestb procedure!
   return/exit
endif
! 
set/format i1 f6.2
@@ ccdtestt ? FF_{p4}_{p1}_{p2}.cat Std_{p4}_{p1}_{p2} ? {shutLOW},{shutHIGH}
if errsum .gt. 0 then
   write/out ****** procedure ccd_standard_test failed:
   write/out ****** problems with ccdtestt procedure!
   return/exit
endif
! 
log/on
log/tof

write/out "     ============================================================"
write/out "     SUMMARY STANDARD TEST CCD#{p1} TAKEN ON {p2} IN {P4} MODE"
write/out "     ============================================================"
write/out " "


write/out "      BIAS"
write/out "      ===="
write/out " "
set/format t6.2
write/out -
     "      5 biases at start test: average = {BIASMEAN} stddev = {BIASSIGM}"
write/out " "
write/out "      There were {hot_pix} hot pixels/columns detected"
write/out " "
write/out " "
write/out "      TRANSFER FUNCTION AND LINEARITY DATA"
write/out "      ===================================="
write/out " "
read/table Std_{p4}_{p1}_{p2} #3 #5 #6 #7 #8 #10 #9
stat/table Std_{p4}_{p1}_{p2} #5
minmax(1) = outputr(1)
minmax(2) = outputr(2)
stat/table Std_{p4}_{p1}_{p2} #6
minmax(1) = minmax(1) + outputr(1)
minmax(2) = minmax(2) + outputr(2)
minmax(1) = minmax(1) * ADUCF / 2
minmax(2) = minmax(2) * ADUCF / 2
set/format f6.0

write/out " "
write/out "      Dynamic range explored: from {minmax(1)} to {minmax(2)} e-"
write/out "      Average count rate measured: {counts} ADU/sec."
write/out " "

set/format t6.2

write/out "      CONV. FACTOR AND RON"
write/out "      ===================="
write/out " "
write/out "      C.Fact = {ADUCF} +/- {errcf} e-/ADU"
write/out " "
write/out "      RON    = {ENOISE} +/- {erren} e- RMS"
write/out " "
write/out " "


write/out "      SHUTTER ERROR"
write/out "      ============="
write/out " "
set/format t6.3
write/out -
" Average shutter error derived from fractional count rate = {ESHUTTER} sec"
write/out " "
write/out " "
set/format t5.2

write/out "      LINEARITY ERROR"
write/out "      ==============="
write/out " "
write/out -
" Average linearity error after shutter correct. = +{linerr(6)}/{linerr(5)} %"
write/out " "
write/out " These errors represent maximum deviation in % from the average"
write/out " count rate after correction for shutter delay"
!
! Create table with data for publication
set/format e12.5 i1
$echo {p2(1:4)} {p2(5:6)} {p2(7:8)} {BIASMEAN} {BIASSIGM} {COUNTS} {ADUCF} {errcf} {ENOISE} {erren} {ESHUTTER} {linerr(6)} {linerr(5)} >Std_{p4}_{p1}_{p2}.tab

 
